@@ -59,10 +59,12 @@
 #include <signal.h>
 
 #include "types.h"
-#include "d_memdebug.h"
 #include "main_resources.h"
 #include "main_globals.h"
 #include "misc.h"
+#include "actions.h"
+
+#include "resource.h"
 
 #ifndef XlibSpecificationRelease
 typedef char *XPointer;
@@ -84,8 +86,6 @@ typedef char *XPointer;
 #endif
 
 #include <errno.h>
-/* BSD 4.3 errno.h does not declare errno */
-extern int errno;
 /* Both error returns are checked for non-blocking I/O. */
 /* Manufacture the other error code if only one exists. */
 #if !defined(EWOULDBLOCK) && defined(EAGAIN)
@@ -164,6 +164,8 @@ static XtResource resources[] = {
        offset(pref_width), XtRImmediate, (XtPointer)1},
     {XtNpreferredHeight, XtCPreferredHeight, XtRDimension, sizeof(Dimension),
        offset(pref_height), XtRImmediate, (XtPointer)1},
+    {XtNsafeDir, XtCSafeDir, XtRBoolean, sizeof(Boolean),
+          offset(safeDir), XtRImmediate, (XtPointer)True },
     {XtNsafer, XtCSafer, XtRBoolean, sizeof(Boolean),
           offset(safer), XtRImmediate, (XtPointer)True },
     {XtNinterpreter, XtCInterpreter, XtRString, sizeof(String),
@@ -197,7 +199,7 @@ static void SetBackground(Widget,Bool);
 static Boolean Setup(Widget);
 static void StartInterpreter(Widget);
 static void StopInterpreter(Widget);
-static void InterpreterFailed(Widget);
+static void InterpreterFailed(Widget,const char *);
 static void ChangeCursor(GhostviewWidget,int);
 
 static XtActionsRec actions[] =
@@ -267,8 +269,7 @@ WidgetClass ghostviewWidgetClass = (WidgetClass)&ghostviewClassRec;
 
 static Pixmap pix = 0;
 
-static void Copy_pixmap(w)
-    Widget w;
+static void Copy_pixmap(Widget w)
 {
    GhostviewWidget gvw = (GhostviewWidget) w;
 
@@ -288,8 +289,8 @@ static void Copy_pixmap(w)
   }      
 
 }
-static void Realize_pixmap(w)
-    Widget w;
+
+static void Realize_pixmap(Widget w)
 {
     if(!pix) {
         Display *dpy = XtDisplay(w);
@@ -305,11 +306,8 @@ static void Realize_pixmap(w)
     } 
 }
 
-static void 
-Redisplay(w, event, region)
-Widget w;
-XEvent *event;
-Region region;
+static void
+Redisplay(Widget w, XEvent *event _GL_UNUSED, Region region _GL_UNUSED)
 {
 
   BEGINMESSAGE(Redisplay)
@@ -327,11 +325,7 @@ Region region;
 /*###################################################################################*/
 
 static void
-Message(w, event, params, num_params)
-   Widget w;
-   XEvent *event;
-   String *params;		/* unused */
-   Cardinal *num_params;	/* unused */
+Message(Widget w, XEvent *event, String *params _GL_UNUSED, Cardinal *num_params _GL_UNUSED)
 {
    GhostviewWidget gvw = (GhostviewWidget) w;
    GhostviewWidgetClass gvc = (GhostviewWidgetClass) XtClass(w);
@@ -363,11 +357,7 @@ Message(w, event, params, num_params)
 }
 
 static void
-action_changeCursor(w, event, params, num_params)
-   Widget w;
-   XEvent *event;
-   String *params;
-   Cardinal *num_params;
+action_changeCursor(Widget w, XEvent *event _GL_UNUSED, String *params, Cardinal *num_params)
 {
    int which=0;
 
@@ -385,9 +375,7 @@ action_changeCursor(w, event, params, num_params)
 /*----------------------------------------------------------*/
 
 static void
-ChangeCursor(gvw, which)
-   GhostviewWidget gvw;
-   int which;
+ChangeCursor(GhostviewWidget gvw, int which)
 {
    Widget w = (Widget) gvw;
    Cursor cursor;
@@ -431,11 +419,7 @@ ChangeCursor(gvw, which)
 /*###################################################################################*/
 
 static void
-Notify(w, event, params, num_params)
-  Widget w;
-  XEvent *event;
-  String *params;
-  Cardinal *num_params;
+Notify(Widget w, XEvent *event, String *params _GL_UNUSED, Cardinal *num_params _GL_UNUSED)
 {
   GhostviewWidget gvw = (GhostviewWidget) w;
   GhostviewReturnStruct ret_val;
@@ -467,8 +451,7 @@ Notify(w, event, params, num_params)
 static Boolean broken_pipe = False;
 
 static SIGVAL
-CatchPipe(i)
-    int i;
+CatchPipe(int i _GL_UNUSED)
 {
     broken_pipe = True;
 #ifdef SIGNALRETURNSINT
@@ -488,15 +471,12 @@ CatchPipe(i)
 /*###################################################################################*/
 
 static void
-Input(client_data, source, id)
-    XtPointer client_data;
-    int *source;
-    XtInputId *id;
+Input(XtPointer client_data, int *source _GL_UNUSED, XtInputId *id _GL_UNUSED)
 {
     Widget w = (Widget) client_data;
     GhostviewWidget gvw = (GhostviewWidget) w;
     int bytes_written;
-    SIGVAL (*oldsig)();
+    SIGVAL (*oldsig)(int);
 
     BEGINMESSAGE(Input)
 
@@ -513,7 +493,7 @@ Input(client_data, source, id)
 		struct record_list *ps_old = gvw->ghostview.ps_input;
 		gvw->ghostview.ps_input = ps_old->next;
 		if (ps_old->close) fclose(ps_old->fp);
-		GV_XtFree((char *)ps_old);
+		XtFree((char *)ps_old);
 	    }
 
 	    /* Have to seek at the beginning of each section */
@@ -541,7 +521,7 @@ Input(client_data, source, id)
 	    }
 	    if (gvw->ghostview.bytes_left > 0 &&
 		gvw->ghostview.buffer_bytes_left == 0) {
-		InterpreterFailed(w);	/* Error occurred */
+		InterpreterFailed(w,"bytes left");	/* Error occurred */
 	    }
 	    gvw->ghostview.input_buffer_ptr = gvw->ghostview.input_buffer;
 	    gvw->ghostview.bytes_left -= gvw->ghostview.buffer_bytes_left;
@@ -563,10 +543,10 @@ Input(client_data, source, id)
 #endif
 	    if (broken_pipe) {
 		broken_pipe = False;
-		InterpreterFailed(w);		/* Something bad happened */
+		InterpreterFailed(w,"broken pope");		/* Something bad happened */
 	    } else if (bytes_written == -1) {
 		if ((errno != EWOULDBLOCK) && (errno != EAGAIN)) {
-		    InterpreterFailed(w);	/* Something bad happened */
+		    InterpreterFailed(w,"write to pipe failed");	/* Something bad happened */
 		}
 	    } else {
 	       gvw->ghostview.buffer_bytes_left -= bytes_written;
@@ -586,10 +566,10 @@ Input(client_data, source, id)
                   b=write(gvw->ghostview.interpreter_input,"\n",1);
                   if (broken_pipe) {
                      broken_pipe = False;
-                     InterpreterFailed(w);		/* Something bad happened */
+                     InterpreterFailed(w,"broken pipe writing NL");		/* Something bad happened */
                   } else if (b == -1) {
                      if ((errno != EWOULDBLOCK) && (errno != EAGAIN)) {
-                        InterpreterFailed(w);	/* Something bad happened */
+                        InterpreterFailed(w,"writing NL to pipe failed");	/* Something bad happened */
                      }
                   }
                }
@@ -614,10 +594,10 @@ Input(client_data, source, id)
            b = write(gvw->ghostview.interpreter_input, peek_buf, PEEK_SIZE);
            if (broken_pipe) {
 	      broken_pipe = False;
-	      InterpreterFailed(w);
+	      InterpreterFailed(w,"broken pipe writing peek buf");
            } else if (b == -1) {
 	      if ((errno != EWOULDBLOCK) && (errno != EAGAIN)) {
-	         InterpreterFailed(w);	/* Something bad happened */
+	         InterpreterFailed(w,"writing peek buf to pipe failed");	/* Something bad happened */
 	      }
 	   }
 	}
@@ -637,10 +617,7 @@ Input(client_data, source, id)
 /*###################################################################################*/
 
 static void
-Output(client_data, source, id)
-    XtPointer client_data;
-    int *source;
-    XtInputId *id;
+Output(XtPointer client_data, int *source, XtInputId *id _GL_UNUSED)
 {
     Widget w = (Widget) client_data;
     GhostviewWidget gvw = (GhostviewWidget) w;
@@ -660,7 +637,7 @@ Output(client_data, source, id)
 	    return;
 	} else if (bytes == -1) {
 	    INFMESSAGE(something bad happened in interpreter_output)
-	    InterpreterFailed(w);		/* Something bad happened */
+	    InterpreterFailed(w,"read stdout from pipe failed");		/* Something bad happened */
             ENDMESSAGE(Output)
 	    return;
 	}
@@ -675,7 +652,7 @@ Output(client_data, source, id)
 	    return;
 	} else if (bytes == -1) {
 	    INFMESSAGE(something bad happened in interpreter_error)
-	    InterpreterFailed(w);		/* Something bad happened */
+	    InterpreterFailed(w,"read stderr from pipe failed");		/* Something bad happened */
             ENDMESSAGE(Output)
 	    return;
 	}
@@ -733,10 +710,7 @@ ClassPartInitialize(class)
 /*###################################################################################*/
 
 static void
-Initialize(request, new, args, num_args)
-    Widget request, new;
-    ArgList args;	/* unused */
-    Cardinal *num_args;	/* unused */
+Initialize(Widget request, Widget new, ArgList args _GL_UNUSED, Cardinal *num_args _GL_UNUSED)
 {
     XGCValues	values;
     XtGCMask	mask;
@@ -799,10 +773,7 @@ Initialize(request, new, args, num_args)
 /*###################################################################################*/
 
 static void
-Realize(w, valueMask, attributes)
-    Widget w;
-    Mask *valueMask;
-    XSetWindowAttributes *attributes;
+Realize(Widget w, Mask *valueMask, XSetWindowAttributes *attributes)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
 
@@ -824,8 +795,7 @@ Realize(w, valueMask, attributes)
 /*###################################################################################*/
 
 static void
-Destroy(w)
-    Widget w;
+Destroy(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
 
@@ -833,7 +803,7 @@ Destroy(w)
     StopInterpreter(w);
     XtReleaseGC(w, gvw->ghostview.gc);
     XtReleaseGC(w, gvw->ghostview.highlight_gc);
-    if (gvw->ghostview.input_buffer) GV_XtFree(gvw->ghostview.input_buffer);
+    XtFree(gvw->ghostview.input_buffer);
     if (gvw->core.background_pixmap != XtUnspecifiedPixmap)
 	XFreePixmap(XtDisplay(w), gvw->core.background_pixmap);
     ENDMESSAGE(Destroy)
@@ -844,9 +814,7 @@ Destroy(w)
 /*###################################################################################*/
 
 static void
-SetBackground(w,clear)
-    Widget w;
-    Bool clear;
+SetBackground(Widget w, Bool clear)
 {
    GhostviewWidget gvw = (GhostviewWidget) w;
 
@@ -885,8 +853,7 @@ SetBackground(w,clear)
 /*###################################################################################*/
 
 static void
-Resize(w)
-  Widget w;
+Resize(Widget w _GL_UNUSED)
 {
   BEGINMESSAGE(Resize)
   ENDMESSAGE(Resize)
@@ -899,7 +866,7 @@ Resize(w)
 /*###################################################################################*/
 
 static Boolean
-SetValues(Widget current, Widget request, Widget new, ArgList unused1, Cardinal *unused2)
+SetValues(Widget current, Widget request, Widget new, ArgList unused1 _GL_UNUSED, Cardinal *unused2 _GL_UNUSED)
 {
     GhostviewWidget cgvw = (GhostviewWidget) current;
     GhostviewWidget rgvw = (GhostviewWidget) request;
@@ -962,6 +929,7 @@ SetValues(Widget current, Widget request, Widget new, ArgList unused1, Cardinal
 	   (cgvw->ghostview.quiet != ngvw->ghostview.quiet)			||
 	   (cgvw->ghostview.infoVerbose != ngvw->ghostview.infoVerbose)			||
 	   (cgvw->ghostview.safer != ngvw->ghostview.safer)			||
+	   (cgvw->ghostview.safeDir != ngvw->ghostview.safeDir)			||
 	   strcmp(cfilename, rfilename)						||
            (cgvw->ghostview.orientation != ngvw->ghostview.orientation)		||
 	   (cgvw->ghostview.use_bpixmap != ngvw->ghostview.use_bpixmap)		||  
@@ -1002,9 +970,7 @@ SetValues(Widget current, Widget request, Widget new, ArgList unused1, Cardinal
 /*###################################################################################*/
 
 static XtGeometryResult 
-QueryGeometry(w, intended, requested)
-Widget w;
-XtWidgetGeometry *intended, *requested;
+QueryGeometry(Widget w, XtWidgetGeometry *intended, XtWidgetGeometry *requested)
 {
     Dimension new_width, new_height;
     Boolean change, width_req, height_req;
@@ -1044,9 +1010,7 @@ XtWidgetGeometry *intended, *requested;
 /*###################################################################################*/
 
 static void
-Layout(w, xfree, yfree)
-    Widget w;
-    Boolean xfree, yfree;
+Layout(Widget w, Boolean xfree, Boolean yfree)
 {
     Dimension width = w->core.width;
     Dimension height = w->core.height;
@@ -1108,10 +1072,8 @@ Layout(w, xfree, yfree)
 static Boolean SetProperty(Widget w, Pixmap bpixmap);
 
 static Boolean
-ComputeSize(w, xfree, yfree, width, height)
-  Widget w;
-  Boolean xfree, yfree;	/* Am I allowed to change width or height */
-  Dimension *width, *height;
+ComputeSize(Widget w, Boolean xfree, Boolean yfree, Dimension *width, Dimension *height)
+  /* xfree, yfree: Am I allowed to change width or height */
 {
   GhostviewWidget gvw = (GhostviewWidget) w;
   Dimension new_width = *width;
@@ -1198,9 +1160,7 @@ static Boolean alloc_error;
 static XErrorHandler oldhandler;
 
 static int
-catch_alloc (dpy, err)
-Display *dpy;
-XErrorEvent *err;
+catch_alloc (Display *dpy, XErrorEvent *err)
 {
     if (err->error_code == BadAlloc) alloc_error = True;
     if (alloc_error) return 0;
@@ -1214,10 +1174,9 @@ XErrorEvent *err;
  * Returns True if a new interpreter was started, False otherwise.
  */
 /*###################################################################################*/
- 
+
 static Boolean
-Setup(w)
-   Widget w;
+Setup(Widget w)
 {
    GhostviewWidget gvw = (GhostviewWidget) w;
    Pixmap bpixmap;
@@ -1293,8 +1252,7 @@ Setup(w)
 
  
 static Boolean
-SetProperty(w, bpixmap)
-   Widget w; Pixmap bpixmap;
+SetProperty(Widget w, Pixmap bpixmap)
 {
    GhostviewWidget gvw = (GhostviewWidget) w;
    GhostviewWidgetClass gvc = (GhostviewWidgetClass) XtClass(w);
@@ -1373,8 +1331,7 @@ SetProperty(w, bpixmap)
  */
 
 static void
-StartInterpreter(w)
-    Widget w;
+StartInterpreter(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     int	std_in[2];
@@ -1410,8 +1367,8 @@ StartInterpreter(w)
     if (gvw->ghostview.disable_start) return;
 
     argv[argc++] = gvw->ghostview.interpreter;
-    if (app_res.antialias) dptr = device = GV_XtNewString(gv_gs_x11_alpha_device);
-    else                   dptr = device = GV_XtNewString(gv_gs_x11_device);
+    if (app_res.antialias) dptr = device = XtNewString(gv_gs_x11_alpha_device);
+    else                   dptr = device = XtNewString(gv_gs_x11_device);
     while (isspace(*dptr)) dptr++;
     while (*dptr) {
 	 argv[argc++] = dptr;
@@ -1419,10 +1376,12 @@ StartInterpreter(w)
 	 if (*dptr) *dptr++ = '\0';
 	 if (argc + 2 >= NUM_ARGS) {
 	    fprintf(stderr, "Too many arguments to interpreter.\n");
+	    clean_safe_tempdir();
 	    exit(EXIT_STATUS_ERROR);
 	 }
 	 while (isspace(*dptr)) dptr++;
     }
+    argv[argc++] = "-dNOPROMPT";
     argv[argc++] = "-dNOPAUSE";
     if (gv_pdf_password)
     {
@@ -1433,7 +1392,7 @@ StartInterpreter(w)
        strcpy(parameter, "-sPDFPassword=");
        password = quote_filename(gv_pdf_password);
        strcat(parameter, password);
-       GV_XtFree(password);
+       XtFree(password);
        argv[argc++] = parameter;
     }
     
@@ -1441,6 +1400,7 @@ StartInterpreter(w)
     if (gvw->ghostview.quiet) argv[argc++] = "-dQUIET";
     if (gvw->ghostview.safer) 
       {
+	argv[argc++] = "-P-";
 	argv[argc++] = "-dSAFER";
 #   ifdef ALLOW_PDF
 	/* The file created by pdf2dsc opens the original
@@ -1451,22 +1411,7 @@ StartInterpreter(w)
       }
 
     if (gvw->ghostview.arguments) {
-	cptr = arguments = GV_XtNewString(gvw->ghostview.arguments);
-	while (isspace(*cptr)) cptr++;
-	while (*cptr) {
-	    argv[argc++] = cptr;
-	    while (*cptr && !isspace(*cptr)) cptr++;
-	    if (*cptr) *cptr++ = '\0';
-	    if (argc + 2 >= NUM_ARGS) {
-		fprintf(stderr, "Too many arguments to interpreter.\n");
-		exit(EXIT_STATUS_ERROR);
-	    }
-	    while (isspace(*cptr)) cptr++;
-	}
-    }
-
-    if (arguments_p == 1) {
-	cptr = arguments = GV_XtNewString(arguments_value);
+	cptr = arguments = XtNewString(gvw->ghostview.arguments);
 	while (isspace(*cptr)) cptr++;
 	while (*cptr) {
 	    argv[argc++] = cptr;
@@ -1474,6 +1419,7 @@ StartInterpreter(w)
 	    if (*cptr) *cptr++ = '\0';
 	    if (argc + 2 >= NUM_ARGS) {
 		fprintf(stderr, "Too many arguments to interpreter.\n");
+		clean_safe_tempdir();
 		exit(EXIT_STATUS_ERROR);
 	    }
 	    while (isspace(*cptr)) cptr++;
@@ -1492,18 +1438,31 @@ StartInterpreter(w)
              argv[argc++] = gvw->ghostview.filename;
           argv[argc++] = "-c";
           argv[argc++] = "quit";
+    } else
+#   endif
+    if (gvw->ghostview.filename && !strcmp(gvw->ghostview.filename,"-")) {
+       /* The ghostscript documentation states that the "-" argument
+	  tells gs that input is coming from a pipe rather than from stdin.
+	  One of the side effects of the "-" argument is that gs does not
+	  flush output at each line of input, but rather accumulates input
+	  and flushes only when the buffer is full.  Since we want gs to
+	  flush output at each line of input, we therefore cannot send "-".
+	  Unfortunately not sending "-" has the side effect that gs no
+	  longer reads correctly through either multiple PostScript files
+	  or PostScript files with multiple pages.  Ah well.
+	  Should gs ever acquire a -DFLUSH argument, then send gs that argument,
+	  and uncomment the following line.  */
+       /* argv[argc++] = "-"; */
     } else {
        argv[argc++] = "-";
     }
-#   else
-       argv[argc++] = "-";
-#   endif
     argv[argc++] = NULL;
 
     if (gvw->ghostview.filename == NULL) {
 	ret = pipe(std_in);
 	if (ret == -1) {
 	    perror("Could not create pipe");
+	    clean_safe_tempdir();
 	    exit(EXIT_STATUS_ERROR);
 	}
     } else if (strcmp(gvw->ghostview.filename, "-")) {
@@ -1511,6 +1470,7 @@ StartInterpreter(w)
           ret = pipe(std_in);
           if (ret == -1) {
              perror("Could not create pipe");
+	     clean_safe_tempdir();
              exit(EXIT_STATUS_ERROR);
           }
 #      else
@@ -1520,17 +1480,32 @@ StartInterpreter(w)
     ret = pipe(std_out);
     if (ret == -1) {
 	perror("Could not create pipe");
+	clean_safe_tempdir();
 	exit(EXIT_STATUS_ERROR);
     }
     ret = pipe(std_err);
     if (ret == -1) {
 	perror("Could not create pipe");
+	clean_safe_tempdir();
 	exit(EXIT_STATUS_ERROR);
     }
 
     gvw->ghostview.changed = False;
-    gvw->ghostview.busy = True;
-    ChangeCursor(gvw,CURSOR_BUSY);
+    if (!((gvw->ghostview.filename && strcmp(gvw->ghostview.filename, "-") == 0)
+       && (gv_gs_arguments && *gv_gs_arguments))) {
+       gvw->ghostview.busy = True;
+       ChangeCursor(gvw,CURSOR_BUSY);
+    }
+
+/*
+    printf("StartInterpreter:\n");
+    printf("%s", argv[0]);
+    for (argc = 1; argv[argc] != NULL; argc++) {
+        printf(" %s", argv[argc]);
+    }
+    printf("\n");
+*/
+
     gvw->ghostview.interpreter_pid = fork();
 
     if (gvw->ghostview.interpreter_pid == 0) { /* child */
@@ -1561,8 +1536,16 @@ StartInterpreter(w)
 	      close(std_in[0]);
 #          endif
 	}
+    	if (gvw->ghostview.safeDir) {
+		if (chdir(gv_safe_gs_workdir) != 0) {
+			sprintf(buf, "Chdir to %s failed",
+					gv_safe_gs_workdir);
+			perror(buf);
+			_exit(EXIT_STATUS_ERROR);
+		}
+    	}
 	execvp(argv[0], argv);
-	sprintf(buf, "Exec of %s failed", argv[0]);
+	sprintf(buf, execOfFailedLabel, argv[0]);
 	perror(buf);
 	_exit(EXIT_STATUS_ERROR);
     } else {
@@ -1584,6 +1567,8 @@ StartInterpreter(w)
 	    gvw->ghostview.interpreter_input = std_in[1];
 	    gvw->ghostview.interpreter_input_id = None;
 	} else if (strcmp(gvw->ghostview.filename, "-")) {
+	    gvw->ghostview.interpreter_input = std_in[1];
+	    gvw->ghostview.interpreter_input_id = None;
 	    close(std_in[0]);
 	}
 	close(std_out[1]);
@@ -1597,8 +1582,8 @@ StartInterpreter(w)
 	    XtAppAddInput(XtWidgetToApplicationContext(w), std_err[0],
 			  (XtPointer)XtInputReadMask, Output, (XtPointer)w);
     }
-    if (arguments) GV_XtFree(arguments);
-    if (device)    GV_XtFree(device);
+    XtFree(arguments);
+    XtFree(device);
     gvw->ghostview.background_cleared=0;
 
     ENDMESSAGE(StartInterpreter)
@@ -1609,8 +1594,7 @@ StartInterpreter(w)
 int restarted = 0;
 
 static void
-StopInterpreter(w)
-    Widget w;
+StopInterpreter(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     BEGINMESSAGE(StopInterpreter)
@@ -1622,6 +1606,7 @@ StopInterpreter(w)
       }
     if (gvw->ghostview.interpreter_input >= 0) 
       {
+	INFMESSAGE(removing interpreter input)
 	close(gvw->ghostview.interpreter_input);
 	gvw->ghostview.interpreter_input = -1;
 	if (gvw->ghostview.interpreter_input_id != None) 
@@ -1636,16 +1621,18 @@ StopInterpreter(w)
 
 	    if (ps_old->close) fclose(ps_old->fp);
 
-	    GV_XtFree((char *)ps_old);
+	    XtFree((char *)ps_old);
 
 	  }
       }
     if (gvw->ghostview.interpreter_output >= 0) {
+        INFMESSAGE(closing interpreter output)
 	close(gvw->ghostview.interpreter_output);
 	gvw->ghostview.interpreter_output = -1;
 	XtRemoveInput(gvw->ghostview.interpreter_output_id);
     }
     if (gvw->ghostview.interpreter_error >= 0) {
+        INFMESSAGE(closing interpreter error)
 	close(gvw->ghostview.interpreter_error);
 	gvw->ghostview.interpreter_error = -1;
 	XtRemoveInput(gvw->ghostview.interpreter_error_id);
@@ -1662,12 +1649,12 @@ StopInterpreter(w)
 /*###################################################################################*/
 
 static void
-InterpreterFailed(w)
-    Widget w;
+InterpreterFailed(Widget w, const char *mesg)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     BEGINMESSAGE(InterpreterFailed)
     StopInterpreter(w);
+    fprintf(stderr, "Error: %s, errno %d\n", mesg, errno);
     XtCallCallbackList(w, gvw->ghostview.message_callback, "Failed");
     ENDMESSAGE(InterpreterFailed)
 }
@@ -1705,15 +1692,7 @@ GhostviewDrawRectangle(w,x,y,width,height)
 #define GV_O_H ((GV_O_L) | (GV_O_S))
 #define GV_O_V ((GV_O_P) | (GV_O_U))
 
-static void GhostviewOrientCoords(gvw,x,y,xP,yP,widthP,heightP,orientP)
-  GhostviewWidget gvw;
-  int x;
-  int y;
-  int *xP;
-  int *yP;
-  int *widthP;
-  int *heightP;
-  int *orientP;
+static void GhostviewOrientCoords(GhostviewWidget gvw, int x, int y, int *xP, int *yP, int *widthP, int *heightP, int *orientP)
 {
   BEGINMESSAGE(GhostviewOrientCoords)
   switch (gvw->ghostview.orientation) {
@@ -1743,13 +1722,7 @@ static void GhostviewOrientCoords(gvw,x,y,xP,yP,widthP,heightP,orientP)
 /* GhostviewCornersPS */
 /*------------------------------------------------------------*/
 
-static void GhostviewCornersPS(gvw,o,pslxP,psrxP,pslyP,psuyP)
-  GhostviewWidget gvw;
-  int o;
-  int *pslxP;
-  int *psrxP;
-  int *pslyP;
-  int *psuyP;
+static void GhostviewCornersPS(GhostviewWidget gvw, int o, int *pslxP, int *psrxP, int *pslyP, int *psuyP)
 {
   if (o & GV_O_P) {
     *pslxP=gvw->ghostview.llx; *psrxP=gvw->ghostview.urx;
@@ -1771,7 +1744,6 @@ static void GhostviewCornersPS(gvw,o,pslxP,psrxP,pslyP,psuyP)
 /*############################################################*/
 
 void
-#if NeedFunctionPrototypes
 GhostviewCoordsXtoPS(
   Widget w,
   int wx,
@@ -1779,14 +1751,6 @@ GhostviewCoordsXtoPS(
   int *psxP,
   int *psyP
 )
-#else
-GhostviewCoordsXtoPS(w,wx,wy,psxP,psyP)
-  Widget w;
-  int wx;
-  int wy;
-  int *psxP;
-  int *psyP;
-#endif
 {
   GhostviewWidget gvw = (GhostviewWidget) w;
   int o,x,y,pslx,psrx,psly,psuy,wrx,wly;
@@ -1798,11 +1762,11 @@ GhostviewCoordsXtoPS(w,wx,wy,psxP,psyP)
   *psyP = (psly + ((float)y*(float)(psuy-psly))/(float)wly + 0.5);
 #if 1
   {
-    int x,y;
+    int xx,yy;
     IIMESSAGE(wx,wy)
     IIMESSAGE(*psxP,*psyP)
-    GhostviewCoordsPStoX(w,*psxP,*psyP,&x,&y);
-    IIMESSAGE(x,y)
+    GhostviewCoordsPStoX(w,*psxP,*psyP,&xx,&yy);
+    IIMESSAGE(xx,yy)
   }
 #endif
   ENDMESSAGE(GhostviewCoordsXtoPS)
@@ -1813,7 +1777,6 @@ GhostviewCoordsXtoPS(w,wx,wy,psxP,psyP)
 /*############################################################*/
 
 void
-#if NeedFunctionPrototypes
 GhostviewCoordsPStoX(
   Widget w,
   int psx,
@@ -1821,14 +1784,6 @@ GhostviewCoordsPStoX(
   int *wxP,
   int *wyP
 )
-#else
-GhostviewCoordsPStoX(w,psx,psy,wxP,wyP)
-  Widget w;
-  int psx;
-  int psy;
-  int *wxP;
-  int *wyP;
-#endif
 {
   GhostviewWidget gvw = (GhostviewWidget) w;
   int o,pslx,psrx,psly,psuy,wrx,wly,*xP,*yP;
@@ -1849,7 +1804,6 @@ GhostviewCoordsPStoX(w,psx,psy,wxP,wyP)
 /*############################################################*/
 
 void
-#if NeedFunctionPrototypes
 GhostviewGetBBofArea (
   Widget   w,
   int x1,
@@ -1858,15 +1812,6 @@ GhostviewGetBBofArea (
   int y2,
   GhostviewReturnStruct* p
 )
-#else
-GhostviewGetBBofArea (w,x1,y1,x2,y2,p)
-  Widget w;
-  int x1;
-  int y1;
-  int x2;
-  int y2;
-  GhostviewReturnStruct* p;
-#endif
 {
   GhostviewWidget gvw = (GhostviewWidget) w;
   int psx1,psy1,psx2,psy2;
@@ -1890,7 +1835,6 @@ GhostviewGetBBofArea (w,x1,y1,x2,y2,p)
 /*############################################################*/
 
 void
-#if NeedFunctionPrototypes
 GhostviewGetAreaOfBB (
   Widget   w,
   int psx1,
@@ -1898,16 +1842,7 @@ GhostviewGetAreaOfBB (
   int psx2,
   int psy2,
   GhostviewReturnStruct* p
-) 
-#else
-GhostviewGetAreaOfBB (w,psx1,psy1,psx2,psy2,p)
-  Widget w;
-  int psx1;
-  int psy1;
-  int psx2;
-  int psy2;
-  GhostviewReturnStruct* p;
-#endif
+)
 {
   int x1,y1,x2,y2;
 
@@ -1929,8 +1864,7 @@ GhostviewGetAreaOfBB (w,psx1,psy1,psx2,psy2,p)
 /*###################################################################################*/
 
 void
-GhostviewClearBackground(w)
-    Widget w;
+GhostviewClearBackground(Widget w)
 {
     BEGINMESSAGE(GhostviewClearBackground)
       if (XtIsRealized(w)) { if(!pix || w != page) SetBackground(w,True); }
@@ -1942,8 +1876,7 @@ GhostviewClearBackground(w)
 /*###################################################################################*/
 
 void
-GhostviewSetup(w)
-    Widget w;
+GhostviewSetup(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     BEGINMESSAGE(GhostviewSetup)
@@ -1958,8 +1891,7 @@ GhostviewSetup(w)
  */
 /*###################################################################################*/
 void
-GhostviewDisableInterpreter(w)
-    Widget w;
+GhostviewDisableInterpreter(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     BEGINMESSAGE(GhostviewDisableInterpreter)
@@ -1976,8 +1908,7 @@ GhostviewDisableInterpreter(w)
 /*###################################################################################*/
 
 void
-GhostviewEnableInterpreter(w)
-    Widget w;
+GhostviewEnableInterpreter(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     BEGINMESSAGE(GhostviewEnableInterpreter)
@@ -1992,20 +1923,12 @@ GhostviewEnableInterpreter(w)
 /*###################################################################################*/
 
 void
-#if NeedFunctionPrototypes
 GhostviewState(
     Widget w,
     Boolean *processflag_p,
     Boolean *busyflag_p,
     Boolean *inputflag_p
 )
-#else
-GhostviewState(w,processflag_p,busyflag_p,inputflag_p)
-    Widget w;
-    Boolean *processflag_p;
-    Boolean *busyflag_p;
-    Boolean *inputflag_p;
-#endif
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     BEGINMESSAGE(GhostviewState)
@@ -2030,8 +1953,7 @@ GhostviewState(w,processflag_p,busyflag_p,inputflag_p)
 /*###################################################################################*/
 
 Boolean
-GhostviewIsInterpreterReady(w)
-    Widget w;
+GhostviewIsInterpreterReady(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     BEGINMESSAGE(GhostviewIsInterpreterReady)
@@ -2053,8 +1975,7 @@ GhostviewIsInterpreterReady(w)
 /*###################################################################################*/
 
 Boolean
-GhostviewIsBusy(w)
-    Widget w;
+GhostviewIsBusy(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     BEGINMESSAGE(GhostviewIsBusy)
@@ -2069,8 +1990,7 @@ GhostviewIsBusy(w)
 /*###################################################################################*/
 
 Boolean
-GhostviewIsInterpreterRunning(w)
-    Widget w;
+GhostviewIsInterpreterRunning(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     BEGINMESSAGE(GhostviewIsInterpreterRunning)
@@ -2096,27 +2016,22 @@ GhostviewIsInterpreterRunning(w)
 /*###################################################################################*/
 
 Boolean
-GhostviewSendPS(w, fp, begin, len, close)
-    Widget w;
-    FILE *fp;
-    gv_off_t begin;
-    gv_off_t len;
-    Bool close;
+GhostviewSendPS(Widget w, FILE *fp, gv_off_t begin, gv_off_t len, Bool doclose)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     struct record_list *ps_new;
 
     if (gvw->ghostview.interpreter_input < 0) return False;
-    ps_new = (struct record_list *) GV_XtMalloc(sizeof (struct record_list));
+    ps_new = (struct record_list *) XtMalloc(sizeof (struct record_list));
     ps_new->fp = fp;
     ps_new->begin = begin;
     ps_new->len = len;
     ps_new->seek_needed = True;
-    ps_new->close = close;
+    ps_new->close = doclose;
     ps_new->next = NULL;
 
     if (gvw->ghostview.input_buffer == NULL) {
-	gvw->ghostview.input_buffer = GV_XtMalloc(GV_BUFSIZ);
+	gvw->ghostview.input_buffer = XtMalloc(GV_BUFSIZ);
     }
 
     if (gvw->ghostview.ps_input == NULL) {
@@ -2148,8 +2063,7 @@ GhostviewSendPS(w, fp, begin, len, close)
 /*###################################################################################*/
 
 Boolean
-GhostviewNextPage(w)
-    Widget w;
+GhostviewNextPage(Widget w)
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
     GhostviewWidgetClass gvc = (GhostviewWidgetClass) XtClass(w);
@@ -2164,8 +2078,11 @@ GhostviewNextPage(w)
     }
 
     if (!gvw->ghostview.busy) {
-	gvw->ghostview.busy = True;
-        ChangeCursor(gvw,CURSOR_BUSY);
+	if (!((gvw->ghostview.filename && strcmp(gvw->ghostview.filename, "-") == 0)
+	   && (gv_gs_arguments && *gv_gs_arguments))) {
+	    gvw->ghostview.busy = True;
+	    ChangeCursor(gvw,CURSOR_BUSY);
+	}
 	event.xclient.type = ClientMessage;
 	event.xclient.display = XtDisplay(w);
 	event.xclient.window = gvw->ghostview.mwin;
@@ -2204,13 +2121,7 @@ GhostviewNextPage(w)
 	}
 
 Boolean
-XmuCvtStringToPalette(dpy, args, num_args, fromVal, toVal, data)
-    Display	*dpy;
-    XrmValue	*args;		/* unused */
-    Cardinal	*num_args;	/* unused */
-    XrmValue	*fromVal;
-    XrmValue	*toVal;
-    XtPointer	*data;		/* unused */
+XmuCvtStringToPalette(Display *dpy, XrmValue *args _GL_UNUSED, Cardinal *num_args _GL_UNUSED, XrmValue *fromVal, XrmValue *toVal, XtPointer *data _GL_UNUSED)
 {
     static XrmQuark		XrmQEmonochrome;
     static XrmQuark		XrmQEgrayscale;
