@@ -29,6 +29,8 @@
 **           GNU Project
 **
 */
+/* for canonicalize_file_name: */
+#define _GNU_SOURCE 1
 #include "ac_config.h"
 
 /*
@@ -68,9 +70,7 @@
 #include <ctype.h>
 
 #include "ac_config.h"
-#ifdef HAVE_INTTYPES_H
-  #include <inttypes.h>
-#endif
+#include <inttypes.h>
 
 #ifndef SEEK_SET
 #   define SEEK_SET 0
@@ -118,7 +118,6 @@
 #include "types.h"
 #include "actions.h"
 #include "callbacks.h"
-#include "d_memdebug.h"
 #include "file.h"
 #include "ps.h"
 #include "doc_misc.h"
@@ -145,16 +144,10 @@
 #define MAP_PAGEVIEW	(1<<4)
 #define MAP_PAGE	(1<<5)
 
-#if NeedFunctionPrototypes
-#   define PT(aaa) aaa
-#else 
-#   define PT(aaa) ()
-#endif
-static Boolean set_new_scale        PT((void));
-static Boolean set_new_orientation  PT((int));
-static Boolean set_new_pagemedia    PT((int));
-static void    layout_ghostview     PT((void));
-#undef PT
+static Boolean set_new_scale        (void);
+static Boolean set_new_orientation  (int);
+static Boolean set_new_pagemedia    (int);
+static void    layout_ghostview     (void);
 
 /*############################################################*/
 /* misc_drawEyeGuide */
@@ -168,10 +161,8 @@ typedef struct
   XtIntervalId timer;
 } EyeGuideDataStruct,*EyeGuideData;
 
-static void 
-misc_catchEyeGuideTimer(client_data, idp)
-  XtPointer client_data;
-  XtIntervalId *idp;
+static void
+misc_catchEyeGuideTimer(XtPointer client_data, XtIntervalId *idp _GL_UNUSED)
 {
   EyeGuideData egd = (EyeGuideData) client_data;
 
@@ -180,11 +171,8 @@ misc_catchEyeGuideTimer(client_data, idp)
   ENDMESSAGE(misc_catchEyeGuideTimer)
 }
 
-void 
-misc_drawEyeGuide(w,d,x,y)
-  Widget w;
-  int d;
-  int x,y;
+void
+misc_drawEyeGuide(Widget w, int d, int x, int y)
 {
   unsigned long t = (unsigned long) 1000;
   static EyeGuideData egd = NULL;
@@ -195,7 +183,7 @@ misc_drawEyeGuide(w,d,x,y)
     return;
   }
   if (!egd) {
-    egd = (EyeGuideData) GV_XtMalloc(sizeof(EyeGuideDataStruct));
+    egd = (EyeGuideData) XtMalloc(sizeof(EyeGuideDataStruct));
     egd->w = w;
     egd->drawn = 0;
     egd->timer = (XtIntervalId) 0;
@@ -204,25 +192,25 @@ misc_drawEyeGuide(w,d,x,y)
     if (egd->timer) XtRemoveTimeOut(egd->timer);
     egd->timer = XtAppAddTimeOut(app_con,t,misc_catchEyeGuideTimer,(XtPointer)egd);
     if (!egd->drawn) {
-      int x1,y1,x2,y2,nx,ny;
+      int x_1,y_1,x_2,y_2,nx,ny;
       Widget clip = XtParent(XtParent(w));
       INFMESSAGE(drawing)
       nx = (int)w->core.x + (int) w->core.border_width;
       ny = (int)w->core.y + (int) w->core.border_width;
-      if (-x>nx) x1 = -x - nx; else x1=0;
-      if (-y>ny) y1 = -y - ny; else y1=0;
-      x2 = x1 + (int)clip->core.width -1; 
-      if (-x<nx) x2 -= nx;
-      if (x2 > w->core.width) x2 = w->core.width;
-      y2 = y1 + (int)clip->core.height - 1;
-      if (-y<ny) y2 -= ny;
-      if (y2 > w->core.height) y2 = w->core.height;
-      if (x1==0) x1 = -1;
-      if (y1==0) y1 = -1;
-      egd->x = x1;
-      egd->y = y1;
-      egd->width = x2-x1;
-      egd->height = y2-y1;
+      if (-x>nx) x_1 = -x - nx; else x_1=0;
+      if (-y>ny) y_1 = -y - ny; else y_1=0;
+      x_2 = x_1 + (int)clip->core.width -1;
+      if (-x<nx) x_2 -= nx;
+      if (x_2 > w->core.width) x_2 = w->core.width;
+      y_2 = y_1 + (int)clip->core.height - 1;
+      if (-y<ny) y_2 -= ny;
+      if (y_2 > w->core.height) y_2 = w->core.height;
+      if (x_1==0) x_1 = -1;
+      if (y_1==0) y_1 = -1;
+      egd->x = x_1;
+      egd->y = y_1;
+      egd->width = x_2-x_1;
+      egd->height = y_2-y_1;
       GhostviewDrawRectangle(w,egd->x,egd->y,egd->width,egd->height);
       egd->drawn=1;
     }
@@ -235,7 +223,7 @@ misc_drawEyeGuide(w,d,x,y)
   }
   if (d & (EYEGUIDE_REMOVE|EYEGUIDE_RESET)) {
     if (egd->timer) XtRemoveTimeOut(egd->timer);
-    GV_XtFree((char*)egd);
+    XtFree((char*)egd);
     egd = NULL;
   }
   ENDMESSAGE(misc_drawEyeGuide)
@@ -248,7 +236,7 @@ misc_drawEyeGuide(w,d,x,y)
 static int pagepos_x,pagepos_y,pagepos_saved=0;
 
 void
-misc_savePagePosition()
+misc_savePagePosition(void)
 {
   int x,y,psx,psy;
   BEGINMESSAGE(misc_savePagePosition)
@@ -266,9 +254,7 @@ misc_savePagePosition()
 /*############################################################*/
 
 int
-misc_restorePagePosition(xP,yP)
-  int *xP;
-  int *yP;
+misc_restorePagePosition(int *xP, int *yP)
 {
   BEGINMESSAGE(misc_restorePagePosition)
   if (pagepos_saved) {
@@ -286,7 +272,7 @@ misc_restorePagePosition(xP,yP)
 /*############################################################*/
 
 void
-misc_resetPagePosition()
+misc_resetPagePosition(void)
 {
   BEGINMESSAGE(misc_resetPagePosition)
   pagepos_saved = 0;
@@ -298,13 +284,11 @@ misc_resetPagePosition()
 /*############################################################*/
 
 void
-misc_setPageMarker(entry,kind,event,check_toc)
-  int entry;
-  int kind; /* 0 = selected, 1 = highlighted , 2 = bring selected in sight*/
-  XEvent *event;
-  Boolean check_toc;
+misc_setPageMarker(int entry, int kind, XEvent *event, Boolean check_toc)
+  /* kind: 0 = selected, 1 = highlighted , 2 = bring selected in sight*/
 {
   int firstvisible, lastvisible;
+  int maxvisible, numentries;
   Boolean b = False;
   INFMESSAGE(misc_setPageMarker)
   if (toc_text && (entry >= 0)) {
@@ -317,7 +301,18 @@ misc_setPageMarker(entry,kind,event,check_toc)
       if (entry<0) return;
     }
     firstvisible = VlistGetFirstVisible(newtoc);
-    if (firstvisible > entry || (entry > 0 && firstvisible >= entry)) {
+    maxvisible = VlistMaxEntriesVisible(newtoc, newtocClip->core.height);
+    numentries = VlistEntries(newtoc);
+    IIMESSAGE(entry,firstvisible)
+    IIMESSAGE(maxvisible,numentries)
+    if (numentries < maxvisible) {
+      /* check if everything fits */
+      /* this can happen if the window size is increased */
+      if (entry != 0) {
+        VlistSetFirstVisible(newtoc, 0);
+        b = True;
+      }
+    } else if (firstvisible > entry || (entry > 0 && firstvisible >= entry)) {
       if (entry > 0)
 	VlistSetFirstVisible(newtoc, entry - 1);
       else
@@ -327,8 +322,13 @@ misc_setPageMarker(entry,kind,event,check_toc)
       /* sadly newtoc does not know it's height, so it cannot be told
        * to made an item visible and we need to trick: */
       lastvisible = VlistEntryOfPosition(newtoc, newtocClip->core.height);
+      IIMESSAGE(lastvisible,maxvisible)
       if (entry > firstvisible && entry >= lastvisible) {
-	VlistSetFirstVisible(newtoc, entry - (lastvisible - firstvisible - 1));
+	int firstentry;
+	firstentry = entry - (lastvisible - firstvisible - 1); /* make the entry second last */
+	if (firstentry > numentries - maxvisible) firstentry = numentries - maxvisible; /* avoid empty holes at the end */
+	if (firstentry < 0) firstentry = 0; /* keep in range */
+	VlistSetFirstVisible(newtoc, firstentry);
 	b = True;
       }
     }
@@ -350,9 +350,7 @@ misc_setPageMarker(entry,kind,event,check_toc)
 /*------------------------------------------------------------*/
 
 static String
-misc_openFile(name, fpP)
-   String name;
-   FILE **fpP;
+misc_openFile(String name, FILE **fpP)
 {
    char *str,*error=NULL;
    FILE *fp=NULL;
@@ -367,7 +365,7 @@ misc_openFile(name, fpP)
          INFMESSAGE(file is not useful)
          str="Invalid file: %s";
          l = strlen(str) + strlen(name) + 1;
-         error = GV_XtMalloc(l*sizeof(char));
+         error = XtMalloc(l*sizeof(char));
          sprintf(error,str,name);
       }
       else if ((fp = fopen(name, "r")) == NULL) {
@@ -388,8 +386,7 @@ misc_openFile(name, fpP)
 /*############################################################*/
 
 String
-misc_testFile(name)
-   String name;
+misc_testFile(String name)
 {
    char *error;
    BEGINMESSAGE(misc_testFile)
@@ -402,59 +399,52 @@ misc_testFile(name)
 /* misc_changeFile */
 /*############################################################*/
 
-String misc_changeFile(name)
-  String name;
+String misc_changeFile(String name)
 {
   FILE *fp=NULL;
   String error=NULL;
   char *p;
   Boolean b = False;
-  char *dir, *q;
 
   BEGINMESSAGE(misc_changeFile)
 
   if (!name) name="";
-  p = GV_XtMalloc((strlen(name)+5)*sizeof(char));
+  p = XtMalloc((strlen(name)+5)*sizeof(char));
   strcpy(p,name);
   if (strcmp(name,"-")) {
     if (!b && file_fileIsNotUseful(p)) sprintf(p,"%s.ps",name);  else b = True;
     if (!b && file_fileIsNotUseful(p)) sprintf(p,"%s.pdf",name); else b = True;
     if (!b)                            strcpy(p,name);
-    dir=file_getDirOfPath(p);
-    if(strcmp(dir,p)) {
-      chdir(dir);
-      /* Strip off directory from p to satisfy GS 8.00 security change */
-      q=p;
-      while(*dir && *dir == *q) {dir++; q++;}
-      strcpy(p,q);
-    }
+    name = canonicalize_file_name(p);
+    if (name)
+	    p = name;
   }
   name = p;
   INFSMESSAGE(trying to open,name)
   error = misc_openFile(name,&fp);
   if (error) {
     ENDMESSAGE(misc_changeFile)
-    GV_XtFree(name);
+    XtFree(name);
     return(error);
   }
 
-  if (gv_filename_old) GV_XtFree(gv_filename_old);
-  if (gv_filename_raw) GV_XtFree(gv_filename_raw);
+  XtFree(gv_filename_old);
+  XtFree(gv_filename_raw);
   gv_filename_old = gv_filename;
   if (gv_filename_dsc) {
     unlink(gv_filename_dsc);
-    GV_XtFree(gv_filename_dsc);
+    XtFree(gv_filename_dsc);
     gv_filename_dsc=NULL;
   }
   if (gv_filename_unc) {
     unlink(gv_filename_unc);
-    GV_XtFree(gv_filename_unc);
+    XtFree(gv_filename_unc);
     gv_filename_unc=NULL;
   }
   if (gv_psfile) fclose(gv_psfile);
   
-  gv_filename = GV_XtNewString(name);
-  gv_filename_raw = GV_XtNewString(name);
+  gv_filename = XtNewString(name);
+  gv_filename_raw = XtNewString(name);
   gv_filename_raw = file_getUsefulName(gv_filename_raw);
   gv_psfile = fp;     
   if (strcmp(name,"-")) {
@@ -463,7 +453,7 @@ String misc_changeFile(name)
     mtime = sbuf.st_mtime;
     INFSMESSAGE(new,gv_filename)
   }
-  GV_XtFree(name);
+  XtFree(name);
   ENDMESSAGE(misc_changeFile)
   return(error);
 }
@@ -472,9 +462,7 @@ String misc_changeFile(name)
 /* close_file */
 /*############################################################*/
 
-String close_file(file,name)
-  FILE *file;
-  String name;
+String close_file(FILE *file, String name)
 {
   char *error=NULL;
 
@@ -483,7 +471,7 @@ String close_file(file,name)
     char *error_close_fail     = "Cannot close file %s\n";
     size_t l;
     l = strlen(error_close_fail) + strlen(name) + 1;
-    error = GV_XtMalloc(l*sizeof(char));
+    error = XtMalloc(l*sizeof(char));
     sprintf(error,error_close_fail,name);
   }
   ENDMESSAGE(close_file)
@@ -499,8 +487,7 @@ String close_file(file,name)
 /*############################################################*/
 
 int
-check_file(mode)
-int mode;
+check_file(int mode)
 {
    int status=0;
    struct stat sbuf;
@@ -548,7 +535,7 @@ int mode;
      }
      NotePopupShowMessage(message);
    }
-   if (gv_filename!=tmpname) GV_XtFree(tmpname);
+   if (gv_filename!=tmpname) XtFree(tmpname);
    ENDMESSAGE(check_file)
    return(r);
 }
@@ -559,8 +546,7 @@ int mode;
 /*------------------------------------------------------------*/
 
 static void
-render_page(gvw)
-    Widget gvw;
+render_page(Widget gvw)
 {
     int i;
 
@@ -636,11 +622,11 @@ render_page(gvw)
 	   XtSetSensitive(w_prevPage, current_page != 0);
 	 }
 	 if (show_nextPage) {
-	   if (current_page == (int)doc->numpages-1) ButtonReset(w_nextPage,NULL,NULL,NULL);
-	   XtSetSensitive(w_nextPage, current_page != (int)doc->numpages-1);
+	   if (current_page == doc->numpages-1) ButtonReset(w_nextPage,NULL,NULL,NULL);
+	   XtSetSensitive(w_nextPage, current_page != doc->numpages-1);
 	 }
 	 XtSetSensitive(prevEntry, current_page != 0);
-	 XtSetSensitive(nextEntry, current_page != (int)doc->numpages-1);
+	 XtSetSensitive(nextEntry, current_page != doc->numpages-1);
        }
        {
          int n = doc ? doc->nummedia : 0;
@@ -665,9 +651,7 @@ render_page(gvw)
 /*############################################################*/
 
 void
-show_page(number,data1)
-   int number;
-   XtPointer data1;
+show_page(int number, XtPointer data1)
 {
    Bool need_layout = False;
    Bool need_setup  = False;
@@ -789,7 +773,7 @@ show_page(number,data1)
                 error = misc_changeFile(filename);
                 if (error) {
 		   NotePopupShowMessage(error);
-		   GV_XtFree(error);
+		   XtFree(error);
  		   ENDMESSAGE(show_page)
                    return;
                 }
@@ -862,10 +846,7 @@ show_page(number,data1)
  */
 /*############################################################*/
 
-static void misc_setSensitive(w,s,b)
-  Widget w;
-  Boolean s;
-  Boolean b;
+static void misc_setSensitive(Widget w, Boolean s, Boolean b)
 {
   if (s) {
     if (!b) ButtonReset(w,NULL,NULL,NULL);
@@ -873,10 +854,7 @@ static void misc_setSensitive(w,s,b)
   }
 }
 
-static void misc_setBitmap(w,s,b)
-  Widget w;
-  Boolean s;
-  Pixmap b;
+static void misc_setBitmap(Widget w, Boolean s, Pixmap b)
 {
   Arg args[1];
   if (s) {
@@ -888,7 +866,7 @@ static void misc_setBitmap(w,s,b)
 }
 
 Boolean
-setup_ghostview()
+setup_ghostview(void)
 {
     Arg args[10];
     Cardinal n;
@@ -904,18 +882,45 @@ setup_ghostview()
     olddoc = doc;
     doc = NULL;
     current_page = NO_CURRENT_PAGE;
-    if (toc_text) GV_XtFree(toc_text);
+    XtFree(toc_text);
     oldtoc_entry_length = toc_entry_length;
     toc_text = NULL;
 
     INFMESSAGE(scanning file for structure information)
     gv_filename_dsc = gv_filename_unc = NULL;
-    doc_scanFile( &gv_psfile,&doc,
+    {
+       char* tmp = malloc(1512);
+       char* src = gv_gs_cmd_scan_pdf;
+       char* dest = tmp;
+       int spaceFound = 0;
+       
+       if (strstr(gv_gs_cmd_scan_pdf, "-P") || !gv_gs_safer)
+          strcpy(tmp, gv_gs_cmd_scan_pdf);
+       else
+       {
+          while (*src)
+          {
+             int isSpace = *src == ' ';
+             *(dest++) = *(src++);
+	     if (!spaceFound && isSpace)
+	     {
+	        strcpy(dest, "-P- -dSAFER -dDELAYSAFER ");
+	        dest+=25;
+		spaceFound = 1;
+	     }
+          }
+	  *dest = 0;
+       }
+
+       doc_scanFile( &gv_psfile,&doc,
 		  gv_filename,
 		  gv_filename_raw,
-		  &gv_filename_dsc,gv_gs_cmd_scan_pdf,
+		  &gv_filename_dsc,tmp,
 		  &gv_filename_unc,gv_uncompress_command,
-		  gv_scanstyle);
+		  gv_scanstyle, gv_gs_safeDir);
+
+       free(tmp);
+    }
     {
       int m;
       m = gv_pagemedia;
@@ -942,7 +947,7 @@ setup_ghostview()
           gv_pagemedia          = m;
        }
     }
-    cb_showTitle(NULL,NULL,NULL);
+    cb_showTitle(toplevel,NULL,NULL);
 
     if (show_date) {
        if (doc && doc->date) {
@@ -991,7 +996,7 @@ setup_ghostview()
         INFMESSAGE(toc available)
 	if (doc->labels_useful) {
 	    for (i = 0; i < doc->numpages; i++) 
-		maxlen = max(maxlen, strlen(doc->pages[i].label));
+		maxlen = max(maxlen, (int)strlen(doc->pages[i].label));
 	} else {
 	    double x;
 	    x = doc->numpages;
@@ -999,7 +1004,7 @@ setup_ghostview()
 	}
 	toc_entry_length = maxlen + 1;
 	toc_length = doc->numpages * toc_entry_length - 1;
-	toc_text = GV_XtMalloc(toc_length + 2); /* include final NULL */
+	toc_text = XtMalloc(toc_length + 2); /* include final NULL */
 
 	for (i = 0, tocp = toc_text; i < doc->numpages;
 	     i++, tocp += toc_entry_length) {
@@ -1033,8 +1038,8 @@ setup_ghostview()
       String s;
       int i=0;
       if (toc_text) {
-	s = (char*)GV_XtMalloc((doc->numpages+1)*sizeof(char));
-	while (i < (int)doc->numpages) {
+	s = (char*)XtMalloc((doc->numpages+1)*sizeof(char));
+	while (i < doc->numpages) {
 	  s[i] = 'p';
 	  i++;
 	}
@@ -1053,10 +1058,11 @@ setup_ghostview()
       }
       XtSetValues(newtoc, args, n);
       ClipWidgetSetCoordinates(newtocClip,0,0);
+      INFMESSAGE(setup_ghostview calling XawScrollbarSetThumb)
       XawScrollbarSetThumb(newtocScroll,
 		      VlistScrollPosition(newtoc),
 		      VlistVisibleLength(newtoc,newtocClip->core.height));
-      GV_XtFree(s);
+      XtFree(s);
     }
 
     misc_setBitmap(w_toggleCurrentPage , show_toggleCurrentPage , app_res.mark_current_bitmap);
@@ -1070,7 +1076,7 @@ setup_ghostview()
     misc_setSensitive(w_printAllPages     , show_printAllPages     , (gv_psfile   != NULL));
     misc_setSensitive(w_checkFile         , show_checkFile         , (gv_filename != NULL));
     misc_setSensitive(w_updateFile        , show_updateFile        , (gv_filename != NULL));
-    misc_setSensitive(w_showThisPage      , show_showThisPage      , (gv_psfile   != NULL));
+    misc_setSensitive(w_showThisPage      , show_showThisPage      , (gv_psfile   != NULL || (gv_gs_arguments && *gv_gs_arguments)));
     misc_setSensitive(w_prevPage          , show_prevPage          , (toc_text    != NULL));
     misc_setSensitive(w_nextPage          , show_nextPage          , (gv_filename != NULL));
     misc_setSensitive(w_toggleCurrentPage , show_toggleCurrentPage , (toc_text    != NULL));
@@ -1086,7 +1092,7 @@ setup_ghostview()
     XtSetSensitive(saveAllEntry,     (gv_psfile   != NULL));
     XtSetSensitive(saveMarkedEntry,  (toc_text    != NULL));
     XtSetSensitive(nextEntry,        (gv_filename != NULL));
-    XtSetSensitive(redisplayEntry,   (gv_psfile   != NULL));
+    XtSetSensitive(redisplayEntry,   (gv_psfile   != NULL || (gv_gs_arguments && *gv_gs_arguments)));
     XtSetSensitive(prevEntry,        (toc_text    != NULL));
     XtSetSensitive(currentEntry,     (toc_text    != NULL));
     XtSetSensitive(oddEntry,         (toc_text    != NULL));
@@ -1099,7 +1105,6 @@ setup_ghostview()
 
 Dimension view_width, view_height, view_border;
 Dimension control_width, control_height;
-Dimension page_width, page_height;
 
 /*------------------------------------------------------------*/
 /* layout_ghostview */
@@ -1175,7 +1180,7 @@ layout_ghostview(void)
 /*############################################################*/
 
 void
-setup_layout_ghostview()
+setup_layout_ghostview(void)
 {
   BEGINMESSAGE(setup_layout_ghostview )
   ENDMESSAGE(setup_layout_ghostview)
@@ -1210,6 +1215,8 @@ set_new_scale(void)
     }
     default_xdpi *= ascale;
     default_ydpi *= ascale;
+    INFIMESSAGE(old scale base,gv_scale_base_current);
+    INFIMESSAGE(new scale base,new_scale_base);
     XtSetArg(args[0], XtNleftBitmap, None);
     if (gv_scale_base_current >=0) XtSetValues(scaleEntry[gv_scale_base_current],args, ONE);
     XtSetArg(args[0], XtNleftBitmap, app_res.selected_bitmap);
@@ -1258,6 +1265,20 @@ set_new_scale(void)
        
        ascale = (float)viewClip->core.width / dx / 72.0 * default_xdpi;
     }
+    else if (fabs(ascale+3) <= 0.001)
+    {
+       int dx = current_urx - current_llx + 1;
+       int dy = current_ury - current_lly + 1;
+       
+       if (gv_orientation == 2 || gv_orientation == 3)
+       {
+          int hlp = dx;
+         dx = dy;
+         dy = hlp;
+       }
+       
+       ascale = (float)viewClip->core.height / dy / 72.0 * default_xdpi;
+    }
     else if (fabs(ascale+2) <= 0.001)
     {
        ascale = gv_ascale;
@@ -1290,13 +1311,12 @@ set_new_scale(void)
 /*------------------------------------------------------------*/
 
 static void
-set_orientationButton_label(orientation)
-   int orientation;
+set_orientationButton_label(int orientation)
 {
    Arg args[1];
    Widget w = portraitEntry;
    String label;
-    
+
    BEGINMESSAGE(set_orientationButton_label)
    if (orientation == O_LANDSCAPE)       w = landscapeEntry;
    else if (orientation == O_UPSIDEDOWN) w = upsidedownEntry;
@@ -1312,15 +1332,14 @@ set_orientationButton_label(orientation)
 /* set_newBitmapIfChanged */
 /*------------------------------------------------------------*/
 
-static void 
-set_newBitmapIfChanged(w,new_bitmap)
-   Widget w;
-   Pixmap new_bitmap;
+static void
+set_newBitmapIfChanged(Widget w, Pixmap new_bitmap)
 {
    Arg args[1];
    Pixmap old_bitmap;
 
    BEGINMESSAGE(set_newBitmapIfChanged)
+   if (!w) return;  /* continuing will eventually cause a core dump */
    XtSetArg(args[0], XtNleftBitmap, &old_bitmap);
    XtGetValues(w, args, ONE);
    if (new_bitmap != old_bitmap) {
@@ -1335,8 +1354,7 @@ set_newBitmapIfChanged(w,new_bitmap)
 /*------------------------------------------------------------*/
 
 static Boolean
-set_new_orientation(pagenumber)
-   int pagenumber;
+set_new_orientation(int pagenumber)
 {
    Boolean changed  = False;
    int from_doc = 0;
@@ -1417,8 +1435,7 @@ set_new_orientation(pagenumber)
 /*------------------------------------------------------------*/
 
 static void
-set_pagemediaButton_label(media_id)
-   int media_id;
+set_pagemediaButton_label(int media_id)
 { 
    String s = NULL;
    Arg args[1];
@@ -1427,6 +1444,7 @@ set_pagemediaButton_label(media_id)
    if (media_id>=0) {
       Widget w;
       if (pagemediaEntry[media_id]) w = pagemediaEntry[media_id];
+      else if (media_id <= 0)       w = NULL;
       else                          w = pagemediaEntry[media_id-1];
       if (!w)
 	  goto out;
@@ -1446,8 +1464,7 @@ out:
 /*------------------------------------------------------------*/
 
 static Boolean
-set_new_pagemedia(pagenumber)
-   int pagenumber;
+set_new_pagemedia(int pagenumber)
 {
    int new_llx,new_lly,new_urx,new_ury;
    Boolean changed = False;
@@ -1579,7 +1596,7 @@ same_document_media(void)
 /* misc_buildPagemediaMenu */
 /*############################################################*/
 
-void misc_buildPagemediaMenu()
+void misc_buildPagemediaMenu(void)
 {
   Widget w;
   int i,num_doc_media;
@@ -1607,8 +1624,8 @@ void misc_buildPagemediaMenu()
   if (doc) num_doc_media = doc->nummedia;
   
   i = gv_num_std_pagemedia + num_doc_media;
-  if (pagemediaEntry) GV_XtFree(pagemediaEntry);
-  pagemediaEntry = (Widget *) GV_XtMalloc(i * sizeof(Widget));
+  XtFree((XtPointer)pagemediaEntry);
+  pagemediaEntry = (Widget *) XtMalloc(i * sizeof(Widget));
   
   if (doc && doc->nummedia) {
     for (i = 0; i < doc->nummedia; i++) {
@@ -1638,10 +1655,7 @@ void misc_buildPagemediaMenu()
 /*------------------------------------------------------------*/
 
 Widget
-build_label_menu(parent, name, label, bitmap)
-    Widget parent;
-    String name, label;
-    Pixmap bitmap;
+build_label_menu(Widget parent, String name, String label, Pixmap bitmap)
 {
     Arg args[5];
     Cardinal n;
@@ -1668,9 +1682,7 @@ build_label_menu(parent, name, label, bitmap)
 /*############################################################*/
 
 int
-catch_Xerror(dpy, err)
-    Display *dpy;
-    XErrorEvent *err;
+catch_Xerror(Display *dpy, XErrorEvent *err)
 {
     BEGINMESSAGE(catch_Xerror)
     if (err->error_code == BadImplementation) {
@@ -1692,15 +1704,14 @@ catch_Xerror(dpy, err)
 /*############################################################*/
 
 char *
-quote_filename (string) 
-     char *string;
+quote_filename (char *string)
 {
     int c;
     char *result, *r, *s;
 
     BEGINMESSAGE(quote_filename)
 
-    result = (char*) GV_XtMalloc((2 * strlen (string) + 1) * sizeof(char));
+    result = (char*) XtMalloc((2 * strlen (string) + 1) * sizeof(char));
 
     for (r = result, s = string; s && (c = *s); s++)
     {
