@@ -43,9 +43,7 @@
 #include <ctype.h>
 #include <math.h>
 
-#ifdef HAVE_INTTYPES_H
-  #include <inttypes.h>
-#endif
+#include <inttypes.h>
 
 #include "paths.h"
 #include INC_X11(Intrinsic.h)
@@ -63,7 +61,6 @@
 
 #include "types.h"
 #include "callbacks.h"
-#include "d_memdebug.h"
 #include "file.h"
 #include "widgets_misc.h"
 #include "main_resources.h"
@@ -76,6 +73,8 @@
 #include "doc_misc.h"
 #include "version.h"
 
+#include "resource.h"
+
 #include <string.h>
 
 static Widget   popup=NULL,optionControl;
@@ -91,12 +90,13 @@ static Widget   forientLabel,forientButton,forientMenu;
 static Widget   scalesLabel,scalesButton,scalesMenu=NULL;
 static Widget   scaleBaseLabel,scaleBaseButton,scaleBaseMenu=NULL;
 
-static String orientations[5] = { "Portrait","Landscape","Upside-Down","Seascape", NULL };
 static int opt_orientation;
 static int opt_pagemedia;
 
-static String popupVerb[5] = { "None", "Errors", "All", NULL };
+String orientations[5] = { 0 };
+String popupVerb[5] = { 0 };
 static String popupVerbExtern[5] = { "Silent", "Errors", "All", NULL };
+static String orientationsExtern[5] = { "Portrait", "Landscape", "Upside-Down", "Seascape", 0 };
 
 static void options_gv_create(void);
 static void options_gv_setOptionsAtEntry(void);
@@ -135,11 +135,12 @@ static void options_gv_setOptionsAtEntry(void)
   widgets_setToggle(watchToggle, (app_res.watch_file ? 1 : 0));
 
   s=NULL;
+
   for (i = 0; orientations[i]; i++) {
-    if (!strcasecmp(app_res.default_orientation,orientations[i]))
+    if (!strcasecmp(app_res.default_orientation,orientationsExtern[i]))
       s = orientations[i];
   }
-  if (!s) s = "Automatic";
+  if (!s) s = automaticLabel;
                                    n=0;
   XtSetArg(args[n], XtNlabel, s);  n++;
   XtSetValues(orientButton, args, n);
@@ -147,7 +148,7 @@ static void options_gv_setOptionsAtEntry(void)
 
   s=NULL;
   for (i = 0; orientations[i]; i++) {
-    if (!strcasecmp(app_res.fallback_orientation,orientations[i]))
+    if (!strcasecmp(app_res.fallback_orientation,orientationsExtern[i]))
       s = orientations[i];
   }
   if (!s) s = orientations[0];
@@ -173,9 +174,7 @@ static void options_gv_setOptionsAtEntry(void)
    options_gv_getScales
 ------------------------------------------------------*/
 
-static void options_gv_getScales(bP,sP)
-  int *sP;
-  int *bP;
+static void options_gv_getScales(int *bP,int *sP)
 {
   Arg args[1];
   Cardinal n;
@@ -206,9 +205,7 @@ static void options_gv_getScales(bP,sP)
    options_gv_cb_apply
 ------------------------------------------------------*/
 
-static void options_gv_cb_apply(w, client_data, call_data)
-   Widget	w;
-   XtPointer	client_data, call_data;
+static void options_gv_cb_apply(Widget w _GL_UNUSED, XtPointer client_data _GL_UNUSED, XtPointer call_data _GL_UNUSED)
 {
    Arg args[5];
    Cardinal n;
@@ -265,11 +262,11 @@ static void options_gv_cb_apply(w, client_data, call_data)
      reopen=True;
    }
 
-   if (s_media) GV_XtFree(app_res.default_pagemedia);
+   if (s_media) XtFree(app_res.default_pagemedia);
 							n=0;
    XtSetArg(args[n], XtNlabel, &app_res.default_pagemedia);n++;
    XtGetValues(mediaButton, args, n);
-   app_res.default_pagemedia = GV_XtNewString(app_res.default_pagemedia);
+   app_res.default_pagemedia = XtNewString(app_res.default_pagemedia);
    i = doc_convStringToPageMedia(NULL,app_res.default_pagemedia);
    if (i != opt_pagemedia) {
      INFMESSAGE(pagemedia changed)
@@ -283,19 +280,25 @@ static void options_gv_cb_apply(w, client_data, call_data)
    }
    s_media = True;
 
-   if (s_fmedia) GV_XtFree(app_res.fallback_pagemedia);
+   if (s_fmedia) XtFree(app_res.fallback_pagemedia);
 							n=0;
    XtSetArg(args[n], XtNlabel, &app_res.fallback_pagemedia);n++;
    XtGetValues(fmediaButton, args, n);
-   app_res.fallback_pagemedia = GV_XtNewString(app_res.fallback_pagemedia);
+   app_res.fallback_pagemedia = XtNewString(app_res.fallback_pagemedia);
    gv_fallback_pagemedia = doc_convStringToPageMedia(NULL,app_res.fallback_pagemedia);
    s_fmedia = True;
 
-   if (s_orient) GV_XtFree(app_res.default_orientation);
+   if (s_orient) XtFree(app_res.default_orientation);
 							n=0;
-   XtSetArg(args[n], XtNlabel, &app_res.default_orientation);n++;
+   XtSetArg(args[n], XtNlabel, &v);n++;
    XtGetValues(orientButton, args, n);
-   app_res.default_orientation = GV_XtNewString(app_res.default_orientation);
+
+   if (!strcmp(v, automaticLabel))   app_res.default_orientation = XtNewString("Automatic");
+   else if (!strcmp(v, orientations[0]))   app_res.default_orientation = XtNewString(orientationsExtern[0]);
+   else if (!strcmp(v, orientations[1]))   app_res.default_orientation = XtNewString(orientationsExtern[1]);
+   else if (!strcmp(v, orientations[2]))   app_res.default_orientation = XtNewString(orientationsExtern[2]);
+   else if (!strcmp(v, orientations[3]))   app_res.default_orientation = XtNewString(orientationsExtern[3]);
+
    i = doc_convStringToDocOrient(app_res.default_orientation);
    if (i != opt_orientation) {
      INFMESSAGE(orientation changed)
@@ -308,11 +311,16 @@ static void options_gv_cb_apply(w, client_data, call_data)
    }
    s_orient = True;
 
-   if (s_forient) GV_XtFree(app_res.fallback_orientation);
+   if (s_forient) XtFree(app_res.fallback_orientation);
 							n=0;
-   XtSetArg(args[n], XtNlabel, &app_res.fallback_orientation);n++;
+   XtSetArg(args[n], XtNlabel, &v);n++;
    XtGetValues(forientButton, args, n);
-   app_res.fallback_orientation = GV_XtNewString(app_res.fallback_orientation);
+
+   if (!strcmp(v, orientations[0]))   app_res.fallback_orientation = XtNewString(orientationsExtern[0]);
+   else if (!strcmp(v, orientations[1]))   app_res.fallback_orientation = XtNewString(orientationsExtern[1]);
+   else if (!strcmp(v, orientations[2]))   app_res.fallback_orientation = XtNewString(orientationsExtern[2]);
+   else if (!strcmp(v, orientations[3]))   app_res.fallback_orientation = XtNewString(orientationsExtern[3]);
+
    gv_fallback_orientation = doc_convStringToDocOrient(app_res.fallback_orientation);
    s_forient = True;
 
@@ -356,14 +364,30 @@ static void options_gv_cb_apply(w, client_data, call_data)
    ENDMESSAGE(options_gv_cb_apply)
 }
 
+static char* orientation2extern(String l)
+{
+   int i;
+   for (i=0; orientations[i]; i++)
+      if (!strcmp(l, orientations[i]))
+          return orientationsExtern[i];
+   if (!strcmp(l, automaticLabel))
+      return "Automatic";
+   return l;
+}
+
+static char* pagemedia2extern(String l)
+{
+   if (!strcmp(l, automaticLabel))
+      return "Automatic";
+   return l;
+}
+
 /*------------------------------------------------------
    options_gv_cb_save
 ------------------------------------------------------*/
 
 static
-void options_gv_cb_save(w, client_data, call_data)
-  Widget	w;
-  XtPointer	client_data, call_data;
+void options_gv_cb_save(Widget w _GL_UNUSED, XtPointer client_data _GL_UNUSED, XtPointer call_data _GL_UNUSED)
 {
   int    argn = 0;
   String argi[30];
@@ -395,16 +419,16 @@ void options_gv_cb_save(w, client_data, call_data)
                                     n=0;
   XtSetArg(args[n], XtNlabel, &l);  n++;
   XtGetValues(mediaButton, args, n);
-  options_setArg(&(argi[argn]),&(argv[argn]),s_pageMedia           ,gv_class       ,l);
+  options_setArg(&(argi[argn]),&(argv[argn]),s_pageMedia           ,gv_class       ,pagemedia2extern(l));
        ++argn;
   XtGetValues(fmediaButton, args, n);
-  options_setArg(&(argi[argn]),&(argv[argn]),s_fallbackPageMedia   ,gv_class       ,l);
+  options_setArg(&(argi[argn]),&(argv[argn]),s_fallbackPageMedia   ,gv_class       ,pagemedia2extern(l));
        ++argn;
   XtGetValues(orientButton, args, n);
-  options_setArg(&(argi[argn]),&(argv[argn]),s_orientation         ,gv_class       ,l);
+  options_setArg(&(argi[argn]),&(argv[argn]),s_orientation         ,gv_class       ,orientation2extern(l));
        ++argn;
   XtGetValues(forientButton, args, n);
-  options_setArg(&(argi[argn]),&(argv[argn]),s_fallbackOrientation,gv_class        ,l);
+  options_setArg(&(argi[argn]),&(argv[argn]),s_fallbackOrientation,gv_class        ,orientation2extern(l));
        ++argn;
 
   options_gv_getScales(&b,&s);
@@ -434,8 +458,8 @@ void options_gv_cb_save(w, client_data, call_data)
 
   options_save(argn,argi,argv);
   while (--argn >=0) {
-    GV_XtFree(argi[argn]); 
-    GV_XtFree(argv[argn]);
+    XtFree(argi[argn]);
+    XtFree(argv[argn]);
   }
 
   ENDMESSAGE(options_gv_cb_save)
@@ -445,7 +469,7 @@ void options_gv_cb_save(w, client_data, call_data)
    options_gv_createScaleMenus
 ######################################################*/
 
-void options_gv_createScaleMenus()
+void options_gv_createScaleMenus(void)
 {
   int i,sn=0,bn=0;
   Boolean change;
@@ -492,7 +516,7 @@ void options_gv_createScaleMenus()
    options_gv_createMediaMenus
 ######################################################*/
 
-void options_gv_createMediaMenus()
+void options_gv_createMediaMenus(void)
 {
   int i;
   Widget w;
@@ -518,7 +542,7 @@ void options_gv_createMediaMenus()
   } else {
     options_createLabeledMenu("fmedia",optionControl,&fmediaLabel,&fmediaButton,&fmediaMenu);
   }
-  w = XtCreateManagedWidget("Automatic",smeBSBObjectClass,mediaMenu,NULL,(Cardinal)0);
+  w = XtCreateManagedWidget(automaticLabel,smeBSBObjectClass,mediaMenu,NULL,(Cardinal)0);
   XtAddCallback(w, XtNcallback,options_cb_changeMenuLabel,NULL); 
   for (i = 0; gv_medias[i]; i++) {
     if (gv_medias[i]->used) {
@@ -536,7 +560,7 @@ void options_gv_createMediaMenus()
     if (gv_medias[i]->used && !strcasecmp(app_res.default_pagemedia,gv_medias[i]->name))
       s = gv_medias[i]->name;
   }
-  if (!s) s = "Automatic";
+  if (!s) s = automaticLabel;
                                    n=0;
   XtSetArg(args[n], XtNlabel, s);  n++;
   XtSetValues(mediaButton, args, n);
@@ -595,7 +619,7 @@ void options_gv_create(void)
    options_gv_createScaleMenus();
    options_gv_createMediaMenus();
    options_createLabeledMenu("orient",optionControl,&orientLabel,&orientButton,&orientMenu);
-     w = XtCreateManagedWidget("Automatic",smeBSBObjectClass,orientMenu,args,n);
+     w = XtCreateManagedWidget(automaticLabel,smeBSBObjectClass,orientMenu,args,n);
      XtAddCallback(w, XtNcallback,options_cb_changeMenuLabel,NULL); 
    options_createLabeledMenu("forient",optionControl,&forientLabel,&forientButton,&forientMenu);
      for (i = 0; orientations[i]; i++) {
