@@ -49,21 +49,28 @@
 
 #include "types.h"
 #include "config.h"
-#include "d_memdebug.h"
 #include "file.h"
 #include "main_resources.h"
 #include "main_globals.h"
 #include "resource.h"
 
+#   include <sys/types.h>
+#   include <sys/stat.h>
+#   include <unistd.h>
+
 # ifndef USER_DEFAULTS
 #   define USER_DEFAULTS "~/.gv"
 # endif
 
+extern char versionCompatibility[];
+
 static String class_resources[] = {
 #   include "gv_class.h"
   NULL
 };
 
+#include "gv-i18n.h"
+
 String intern_resources[] = {
 #   include INTERN_RESOURCES_H
   NULL
@@ -98,6 +105,10 @@ static String resource_style_file  = NULL;
 static String resource_ad_file     = NULL;
 static char* resource_mergeFileIntoDatabase(XrmDatabase*,char*);
 
+
+extern int debug_p;
+
+
 /*#######################################################
   resource_freeData
   #######################################################*/
@@ -105,10 +116,10 @@ static char* resource_mergeFileIntoDatabase(XrmDatabase*,char*);
 void resource_freeData()
 {
   BEGINMESSAGE(resource_freeData)
-    if (resource_system_file) GV_XtFree(resource_system_file);
-  if (resource_user_file)   GV_XtFree(resource_user_file);
-  if (resource_style_file)  GV_XtFree(resource_style_file);
-  if (resource_ad_file)     GV_XtFree(resource_ad_file);
+  XtFree(resource_system_file);
+  XtFree(resource_user_file);
+  XtFree(resource_style_file);
+  XtFree(resource_ad_file);
   ENDMESSAGE(resource_freeData)
     }
 
@@ -121,15 +132,19 @@ resource_buildDatabase (
                         Display *display,
                         char *app_class,
                         char *app_name,
-                        int *argcP,
-                        char **argv)
+                        int *argcP _GL_UNUSED,
+                        char **argv _GL_UNUSED)
 {
   XrmDatabase gvdb = XtDatabase(display);
   XrmDatabase db = NULL;
   String *sP;
   String s,t, rpath;
   String tildeGv;
+  char* locale;
+  int i18n;
   char tmp[GV_MAX_FILENAME_LENGTH];
+  char locale1[100], locale2[100], locale3[100];
+  char loc_lang[100], loc_terr[100], loc_cs[100], loc_mod[100];
 
   BEGINMESSAGE(resource_buildDatabase)
 
@@ -141,7 +156,7 @@ resource_buildDatabase (
   /* ### system resources ################# */
   INFMESSAGE(checking for system resources)
 
-  rpath = GV_XtMalloc (strlen (GV_LIBDIR) + strlen ("/gv_system.ad") + 1);
+  rpath = XtMalloc (strlen (GV_LIBDIR) + strlen ("/gv_system.ad") + 1);
   rpath[0] = '\0';
   strcat (rpath, GV_LIBDIR);
   strcat (rpath, "/gv_system.ad");
@@ -160,9 +175,9 @@ resource_buildDatabase (
   INFMESSAGE(checking for user resources)
 
   strcpy(tmp,USER_DEFAULTS);
-  file_translateTildeInPath(tmp);
+  file_translateTildeInPath(tmp, sizeof(tmp));
   if (!file_fileIsNotUseful(tmp)) {
-    s = GV_XtNewString(tmp);
+    s = XtNewString(tmp);
     tildeGv = USER_DEFAULTS;
     haveXUSERFILESEARCHPATH = 0;
   } else {
@@ -178,9 +193,229 @@ resource_buildDatabase (
     resource_user_file = s;
     resource_user_file_symb = tildeGv;
   } else {
-    resource_user_file = GV_XtNewString(tmp);
+    resource_user_file = XtNewString(tmp);
   }
 
+  /* ### Loading localisation ### */
+  i18n = 0;
+  locale = 0;
+  locale = getenv("LC_ALL");
+  if ( !locale || !*locale )
+     locale = getenv("LC_MESSAGES");
+  if ( !locale || !*locale )
+     locale = getenv("LANG");
+  if ( !locale || !*locale )
+     locale = "C";
+
+  {
+     char* cL, * cP;
+
+     cP = loc_lang;
+     cL = locale;
+  
+     *loc_terr = 0;
+     *loc_cs = 0;
+     *loc_mod = 0;
+     
+     while (*cL)
+     {
+        if ( *cL == '_' ) { *cP = 0; cP = loc_terr; }
+        if ( *cL == '.' ) { *cP = 0; cP = loc_cs; }
+        if ( *cL == '@' ) { *cP = 0; cP = loc_mod; }
+	*cP++ = *cL++;
+     }
+     *cP = 0;
+  }
+
+  s = resource_getResource(db,app_class,app_name, "international",0);
+  if (s == NULL || !strcasecmp(s, "False"))
+  {
+     sprintf(locale1, "noint:%s%s", loc_lang, loc_terr);
+     sprintf(locale2, "noint:%s", loc_lang);
+     strcpy(locale3, "C");
+  }
+  else
+  {
+     strcpy(locale1, locale);
+     sprintf(locale2, "%s%s%s", loc_lang, loc_terr, loc_cs);
+     sprintf(locale3, "%s%s", loc_lang, loc_cs);
+  }
+
+  if (debug_p)
+  {
+     printf("Locale1=%s\n", locale1);
+     printf("Locale2=%s\n", locale2);
+     printf("Locale3=%s\n", locale3);
+  }
+
+  if (!i18n)
+  {
+     struct stat buf;
+     
+     strcpy(tmp,USER_DEFAULTS "-");
+     strcat(tmp, locale1);
+     file_translateTildeInPath(tmp, sizeof(tmp));
+     if (!stat(tmp, &buf))
+     {
+        i18n = 1;
+        XrmCombineFileDatabase(tmp,&db,True);
+     }
+  }
+  
+  if (!i18n)
+  {
+     struct stat buf;
+     
+     strcpy(tmp,GV_LIBDIR "/nls/");
+     strcat(tmp, locale1);
+     strcat(tmp, ".dat");
+     if (!stat(tmp, &buf))
+     {
+        i18n = 1;
+        XrmCombineFileDatabase(tmp,&db,True);
+     }
+  }
+
+  if (!i18n)
+  {
+     sP = getI18N(locale1);
+     if (sP)
+     {
+        i18n = 1;
+	while (*sP) XrmPutLineResource(&db,*sP++);
+     }
+  }
+
+
+
+
+
+
+
+
+
+
+
+
+
+  if (!i18n)
+  {
+     struct stat buf;
+     
+     strcpy(tmp,USER_DEFAULTS "-");
+     strcat(tmp, locale2);
+     file_translateTildeInPath(tmp, sizeof(tmp));
+     if (!stat(tmp, &buf))
+     {
+        i18n = 1;
+        XrmCombineFileDatabase(tmp,&db,True);
+     }
+  }
+  
+  if (!i18n)
+  {
+     struct stat buf;
+     
+     strcpy(tmp,GV_LIBDIR "/nls/");
+     strcat(tmp, locale2);
+     strcat(tmp, ".dat");
+     if (!stat(tmp, &buf))
+     {
+        i18n = 1;
+        XrmCombineFileDatabase(tmp,&db,True);
+     }
+  }
+
+  if (!i18n)
+  {
+     sP = getI18N(locale2);
+     if (sP)
+     {
+        i18n = 1;
+	while (*sP) XrmPutLineResource(&db,*sP++);
+     }
+  }
+
+
+
+
+
+
+
+
+  if (!i18n)
+  {
+     struct stat buf;
+     
+     strcpy(tmp,USER_DEFAULTS "-");
+     strcat(tmp, locale3);
+     file_translateTildeInPath(tmp, sizeof(tmp));
+     if (!stat(tmp, &buf))
+     {
+        i18n = 1;
+        XrmCombineFileDatabase(tmp,&db,True);
+     }
+  }
+  
+  if (!i18n)
+  {
+     struct stat buf;
+     
+     strcpy(tmp,GV_LIBDIR "/nls/");
+     strcat(tmp, locale3);
+     strcat(tmp, ".dat");
+     if (!stat(tmp, &buf))
+     {
+        i18n = 1;
+        XrmCombineFileDatabase(tmp,&db,True);
+     }
+  }
+
+  if (!i18n)
+  {
+     sP = getI18N(locale3);
+     if (sP)
+     {
+        i18n = 1;
+	while (*sP) XrmPutLineResource(&db,*sP++);
+     }
+  }
+
+
+
+
+  orientations[0] = resource_getResource(db,app_class,app_name, "portrait.Label",0);
+  orientations[1] = resource_getResource(db,app_class,app_name, "landscape.Label",0); 
+  orientations[2] = resource_getResource(db,app_class,app_name, "upsidedown.Label",0);
+  orientations[3] = resource_getResource(db,app_class,app_name, "seascape.Label",0);
+
+  popupVerb[0] = resource_getResource(db,app_class,app_name, "gs.errors.none.Label",0);
+  popupVerb[1] = resource_getResource(db,app_class,app_name, "gs.errors.errors.Label",0);
+  popupVerb[2] = resource_getResource(db,app_class,app_name, "gs.errors.all.Label",0);
+
+  confirm_quit_styles[0] = resource_getResource(db,app_class,app_name, "quitstyle.never.Label",0);
+  confirm_quit_styles[1] = resource_getResource(db,app_class,app_name, "quitstyle.whenprocessing.Label",0);
+  confirm_quit_styles[2] = resource_getResource(db,app_class,app_name, "quitstyle.always.Label",0);
+
+  title_styles[0] = resource_getResource(db,app_class,app_name, "title.notitle.Label",0);
+  title_styles[1] = resource_getResource(db,app_class,app_name, "title.documenttitle.Label",0);
+  title_styles[2] = resource_getResource(db,app_class,app_name, "title.filename.Label",0);
+
+  automaticLabel = resource_getResource(db,app_class,app_name, "automatic.Label",0);
+  saveCurrentPageLabel = resource_getResource(db,app_class,app_name, "strings.saveCurrentPage",0);
+  saveMarkedPagesLabel = resource_getResource(db,app_class,app_name, "strings.saveMarkedPages",0);
+  saveDocumentLabel = resource_getResource(db,app_class,app_name, "strings.saveDocument",0);
+  saveAsPDFLabel = resource_getResource(db,app_class,app_name, "strings.saveAsPDF",0);
+  openFileLabel = resource_getResource(db,app_class,app_name, "strings.openFile",0);
+  passwordPromptLabel = resource_getResource(db,app_class,app_name, "strings.passwordPrompt",0);
+  passwordRequiredLabel = resource_getResource(db,app_class,app_name, "strings.passwordRequired",0);
+  quitConfirmLabel = resource_getResource(db,app_class,app_name, "strings.quitConfirm",0);
+  putTexCommandLabel = resource_getResource(db,app_class,app_name, "strings.putTexCommand",0);
+  texCommandLabel = resource_getResource(db,app_class,app_name, "strings.texCommand",0);
+  stillInProgressLabel = resource_getResource(db,app_class,app_name, "strings.stillInProgress",0);
+  execOfFailedLabel = resource_getResource(db,app_class,app_name, "strings.execOfFailed",0);
+  copyrightTranslationLabel = resource_getResource(db,app_class,app_name, "strings.copyrightTranslation",0);
+
   /* ### command line resources ################# */
 
   /*
@@ -206,6 +441,14 @@ resource_buildDatabase (
     {
       resource_putResource (&db, app_name, ".gsSafer", "False");
     }
+  if (safedir_p)
+    {
+      resource_putResource (&db, app_name, ".gsSafeDir", "True");
+    }
+  if (nosafedir_p)
+    {
+      resource_putResource (&db, app_name, ".gsSafeDir", "False");
+    }
   if (pixmap_p)
     {
       resource_putResource (&db, app_name, ".useBackingPixmap", "True");
@@ -225,22 +468,22 @@ resource_buildDatabase (
   if (spartan_p)
     {
       char *spartan_filename = (char *)
-	GV_XtMalloc (strlen(GV_LIBDIR) + strlen ("/gv_spartan.dat") + 1);
+	XtMalloc (strlen(GV_LIBDIR) + strlen ("/gv_spartan.dat") + 1);
       spartan_filename[0] = '\0';
       strcat(spartan_filename, GV_LIBDIR);
       strcat(spartan_filename, "/gv_spartan.dat");
       resource_putResource (&db, app_name, ".style", spartan_filename);
-      GV_XtFree (spartan_filename);
+      XtFree (spartan_filename);
     }
   if (widgetless_p)
     {
       char *widgetless_filename = (char *)
-	GV_XtMalloc (strlen(GV_LIBDIR) + strlen ("/gv_widgetless.dat") + 1);
+	XtMalloc (strlen(GV_LIBDIR) + strlen ("/gv_widgetless.dat") + 1);
       widgetless_filename[0] = '\0';
       strcat(widgetless_filename, GV_LIBDIR);
       strcat(widgetless_filename, "/gv_widgetless.dat");
       resource_putResource (&db, app_name, ".style", widgetless_filename);
-      GV_XtFree (widgetless_filename);
+      XtFree (widgetless_filename);
     }
   if (quiet_p)
     {
@@ -392,10 +635,10 @@ resource_buildDatabase (
     } else s = t;
 #   endif
     if (s) {
-      s = GV_XtNewString(s);
+      s = XtNewString(s);
       resource_style_file = s;
     }
-    if (t) GV_XtFree(t);
+    XtFree(t);
   }
 
   /* ### ad resources ######################## */
@@ -409,10 +652,10 @@ resource_buildDatabase (
       s=NULL;
     } else s = t;
     if (s) {
-      s = GV_XtNewString(s);
+      s = XtNewString(s);
       resource_ad_file = s;
     }
-    if (t) GV_XtFree(t);
+    XtFree(t);
   }
 
   ENDMESSAGE(resource_buildDatabase)
@@ -557,13 +800,38 @@ int resource_checkResources(app_name,v,vc)
     if (resource_style_file)  fprintf(stderr,"    %s\n",resource_style_file);
     if (resource_ad_file)     fprintf(stderr,"    %s\n",resource_ad_file);
     fprintf(stderr,"belongs to an older version of gv and cannot be used.\n");
-    fprintf(stderr,"Please remove or update the outdated file.\n");
     if (resource_user_file) fprintf(stderr,"Quite probably your %s is too old.\n", resource_user_file_symb);
     if (resource_user_file) {
        if (haveXUSERFILESEARCHPATH)
-          fprintf(stderr,"Running gv-update-userconfig %s should help\nby removing all incompatible resources.\n", resource_user_file);
+       {
+          if (access(resource_user_file, W_OK))
+	  {
+             fprintf(stderr, "For a correct installation the following restrictions must be met:\n\n");
+	     fprintf(stderr, "* Every directory contained in XUSERFILESEARCHPATH where the system also\nsearches for app-defaults does not contain a file 'GV'.\n\n");
+	     
+	     fprintf(stderr, "* GV is contained just in direectories of XUSERFILESEARCHPATH where the system\ndoes not search for app-defaults, and the GV.version resource is contained in\nthis GV file (this file is used instead of ~/.gv if ~/.gv is not present).\n\n");
+	     
+	     fprintf(stderr, "* The systems app-defaults directories which are not comntained in\nXUSERFILESEARCHPATH may contain a GV file.If so, it SHOULD not contain a\n'GV.version' resource.\n\n");
+	     
+	     fprintf(stderr, "If necessary, ask your system administrator for assistence.\n\n");
+	     
+	     fprintf(stderr,"A workaround to make GNU gv working is\nto provide ~/.gv by running 'touch ~/.gv && gv-update-userconfig'.\n");
+
+          }
+	  else
+	  {
+             fprintf(stderr,"Please remove or update the outdated file.\n");
+             fprintf(stderr,"Running gv-update-userconfig %s should help\nby removing all incompatible resources.\n", resource_user_file);
+	     
+             fprintf(stderr,"Another option that non-admins can do to make GNU gv working is\nto provide ~/.gv by running 'touch ~/.gv && gv-update-userconfig'.\n");
+
+          }
+       }
        else
+       {
+          fprintf(stderr,"Please remove or update the outdated file.\n");
           fprintf(stderr,"Running gv-update-userconfig should help\nby removing all incompatible resources.\n");
+       }
     }
     r=0;
   }
@@ -583,7 +851,7 @@ char *resource_userDefaultsFile()
     if      (resource_ad_file)   s = resource_ad_file;
     else if (resource_user_file) s = resource_user_file;
     else                         s = USER_DEFAULTS;
-  s = GV_XtNewString(s);
+  s = XtNewString(s);
   return(s);
   ENDMESSAGE(resource_userResourceFile)  
     }
@@ -598,7 +866,6 @@ static char* resource_mergeFileIntoDatabase(dbP,name)
      char *name;
 {
   char tmp[GV_MAX_FILENAME_LENGTH];
-  int useful=0;
 
   BEGINMESSAGE(resource_mergeFileIntoDatabase)
 
@@ -608,38 +875,34 @@ static char* resource_mergeFileIntoDatabase(dbP,name)
     }
 
   strcpy(tmp,name);
-  file_translateTildeInPath(tmp);
+  file_translateTildeInPath(tmp, sizeof(tmp));
 
-  if (file_fileIsNotUseful(tmp)) {
+  if (tmp[0] != '/') {
+    fprintf(stderr, "Ignoring resource file '%s'='%s' as no absolute path!\n",
+		    name, tmp);
+    name = NULL;
+  } else if (file_fileIsNotUseful(tmp)) {
     INFSMESSAGE(not useful,tmp)
-      if (name != file_locateFilename(name)) useful=-1;
-      else {
-
-	sprintf(tmp,"~/%s",name);
-	file_translateTildeInPath(tmp);
-
-      }
-  } else useful=1;
-
-  if (!useful) {
-
-    if (!useful && file_fileIsNotUseful(tmp)) {
-      INFSMESSAGE(not useful,tmp)
-	sprintf(tmp,"%s%s",GV_LIBDIR,name);
-    } else useful=1;
-
-    if (!useful && file_fileIsNotUseful(tmp)) {
-      INFSMESSAGE(not useful,tmp)
-	INFMESSAGE(giving up)
-	} else useful=1;
-  }
-
-  if (useful==1) {
+    name = NULL;
+  } else {
     INFSMESSAGE(merging,tmp)
       XrmCombineFileDatabase(tmp,dbP,True);
-    name = GV_XtNewString(tmp);
+    name = XtNewString(tmp);
   }
-  else name=NULL;
   ENDMESSAGE(resource_mergeFileIntoDatabase)
     return(name);
 }
+
+String automaticLabel;
+String saveCurrentPageLabel;
+String saveMarkedPagesLabel;
+String saveDocumentLabel;
+String saveAsPDFLabel;
+String openFileLabel;
+String passwordPromptLabel;
+String passwordRequiredLabel;
+String quitConfirmLabel;
+String putTexCommandLabel, texCommandLabel;
+String stillInProgressLabel;
+String execOfFailedLabel;
+String copyrightTranslationLabel;
